import 'generator.dart' show Generator;

// Trieda reprezentuje zavodníka
class Zavodnik {
  // meno a priezvisko zavodníka
  String _meno;
  // kondícia závodníka, čím vyššie číslo, tým nižšia kondícia
  double _kondicia;
  // presnosť v strelbe, čím vyššie číslo, tým vyššia presnosť
  double _presnost;
  // celkový čas v sekundách
  double celkovy_cas_behu = 0.0;
  // čas v minútach
  double _minuty_medzicas;
  // čas v sekundách
  double _sekundy_medzicas;
  // čas v minútach
  int trestne_minuty_spolu = 0;
  // posledná správa
  String _sprava;
  // hod kockou
  int _hod_kockou;

  // konštruktor
  Zavodnik(this._meno, this._presnost);
  // závodník ubehne 1 km s nastavenou zložitosťou terénu
  void beh(double zlozitost, int km) {
    Generator gz = new Generator();
    // deklarácia premenných porebných na výpočty času
    double cas, minuty_cas, sekundy_cas;
    // generovanie kondície, po každom km sa môže zmeniť náročnosť terénu a sila závodníka
    _kondicia = gz.generuj_kondicia();
    // hod kockou
    _hod_kockou = gz.generuj_hod_kockou();
    switch (_hod_kockou) {
      case 1:
        cas = 1000 * 1.0 * zlozitost * _kondicia; // 1000 m => 1 km
        break;
      case 2:
        cas = 1000 * 1.05 * zlozitost * _kondicia; // 1.0 - 1.25 => náhoda podľa hodu
        break;
      case 3:
        cas = 1000 * 1.1 * zlozitost * _kondicia; // kondícia je jasná
        break;
      case 4:
        cas = 1000 * 1.15 * zlozitost * _kondicia; // zložitosť je jasná
        break;
      case 5:
        cas = 1000 * 1.2 * zlozitost * _kondicia;
        break;
      case 6:
        cas = 1000 * 1.25 * zlozitost * _kondicia;
        break;
    }
    minuty_cas = cas / 60; // sekundy na minúty
    sekundy_cas = cas - (minuty_cas.floor() * 60); // zvyšné sekundy
    celkovy_cas_behu += cas; // načítavanie celkového času preteku
    _minuty_medzicas = celkovy_cas_behu / 60; // sekundy na minúty
    _sekundy_medzicas = celkovy_cas_behu - (_minuty_medzicas.floor() * 60); // zvyšné sekundy
    // nastaví správu
    nastav_spravu('$_meno presiel $km. km za ${minuty_cas.floor()} minut a ${sekundy_cas.toStringAsFixed(1)} sekund\n'
    'v medzicase ${_minuty_medzicas.floor()} minut a ${_sekundy_medzicas.toStringAsFixed(1)} sekund.\n');
    print(vrat_poslednu_spravu());
  }
  // 5  výstrelov závodníka
  int vystrel(int pocitadlo, int vyhodnotenie) {
    // inštancia na použitie hodu kockou
    Generator gz = new Generator();
    // strelba v leze alebo v stoji
    double narocnost;
    // vynulovanie trestných minút
    int trestne_min = 0;
    // príprava na grafické výstrely
    String pal_vysledok, jednotlive_vystrely = '';
    // rozdelenie na strelbu v ľahu a v stoji
    if (pocitadlo % 2 == 0) {
      print('Zacina strelba v stoji:');
      narocnost = 1.2;
    }
    else {
      print('Zacina strelba v leze:');
      narocnost = 1.0;
    }
    // 5 výstrelov
    for (var j = 0; j < 5; j++) {
      // nastavenie neúspechu
      int vysledok = 1;
      _hod_kockou = gz.generuj_hod_kockou();
      pal_vysledok = '';
      // výpočet trestných minút
      switch (_hod_kockou) {
        case 1: case 2: case 3: case 4: case 5: case 6:
          vysledok = (_hod_kockou * _presnost > narocnost) ? 0 : 1;
          trestne_min += vysledok;
          break;
      }
      // uloženie jednotlivých výstrelov do stringu
      for (var k = 0; k < 5; k++) {
        pal_vysledok += k < j ? jednotlive_vystrely[k] : k == j ? vysledok.toString() : '1';
      }
      jednotlive_vystrely = pal_vysledok;
    }
    // načítavanie trestných minút
    trestne_minuty_spolu += trestne_min;
    // podľa nastavenej hodnoty nastaví správu pre hru alebo vyhodnotenie
    if (vyhodnotenie == 0) {
      // pre hru
      nastav_spravu(
          '$_meno vysledok strelby:\n\n${vystrel_graficky(pal_vysledok)}\n'
              'Trestne minuty za $pocitadlo. strelbu: $trestne_min min.\n'
              'Trestne minuty spolu: $trestne_minuty_spolu min\n');
      print(vrat_poslednu_spravu());
    }
    else {
      // pre vyhodnotenie
      nastav_spravu(
          '$_meno vysledok strelby:\n\n${vystrel_graficky(pal_vysledok)}\n'
              'Netrafene terce: $trestne_min.\n');
      print(vrat_poslednu_spravu());
    }
    return trestne_min;
  }
  // grafické stvárnenie výsledku strelby
  String vystrel_graficky(String vysledok) {
    // deklarácia a definícia pomocných stringov jednotlivých riadkov výpisu
    String n1_6 = '     ***     ';
    String n2_5 = '  *********  ';
    String n3_4 = ' *********** ';
    String t1_6 = '     * *     ';
    String t2_5 = '  *       *  ';
    String t3_4 = ' *         * ';
    String test1 = '';
    String test2 = '';
    String test3 = '';
    String test4 = '';
    String test5 = '';
    String test6 = '';
    // uloženie výsledkov do riadkov výpisu
    for (var i = 0; i < 5; i++) {
      if (vysledok[i] == '1') {
        test1 += n1_6;
        test2 += n2_5;
        test3 += n3_4;
        test4 += n3_4;
        test5 += n2_5;
        test6 += n1_6;
      }
      else {
        test1 += t1_6;
        test2 += t2_5;
        test3 += t3_4;
        test4 += t3_4;
        test5 += t2_5;
        test6 += t1_6;
      }
    }
    // vráti grafický výpis
    return '$test1\n$test2\n$test3\n$test4\n$test5\n$test6\n';
  }
  // výpis dosiahnutého času
  double ciel_vysledok() {
    nastav_spravu('$_meno bol v cieli za ${_minuty_medzicas.floor()} minut a ${_sekundy_medzicas.toStringAsFixed(1)} sekund,\n'
        'za strelbu dostal $trestne_minuty_spolu trestnych minut.\n'
        'Celkovy cas je teda ${(_minuty_medzicas + trestne_minuty_spolu).floor()} minut a ${_sekundy_medzicas.toStringAsFixed(1)} sekund\n\n');
    print(vrat_poslednu_spravu());
    return celkovy_cas_behu + (trestne_minuty_spolu * 60);
  }
  // nastaví správu
  void nastav_spravu(String _sprava) {
    this._sprava = _sprava;
  }
  // vráti poslednú správu
  String vrat_poslednu_spravu() => _sprava;
  /// preťažený toString()
  toString() => _meno;
}
